#include <Arduino.h>
#include "ADC4_lite.h"
// modified version from T4 analog.c
// 0x80 bit - Only on ADC2
// 0x40 bit - only on ADC1
const uint8_t t4_pin_to_channel[] = { 
  7,  // 0/A0  AD_B1_02
  8,  // 1/A1  AD_B1_03
  12, // 2/A2  AD_B1_07
  11, // 3/A3  AD_B1_06
  6,  // 4/A4  AD_B1_01
  5,  // 5/A5  AD_B1_00
  15, // 6/A6  AD_B1_10
  0,  // 7/A7  AD_B1_11
  13, // 8/A8  AD_B1_08
  14, // 9/A9  AD_B1_09
  1,  // 24/A10 AD_B0_12 
  2,  // 25/A11 AD_B0_13
  0x80+3,  // 26/A12 AD_B1_14 - only on ADC2, 3
  0x80+4,  // 27/A13 AD_B1_15 - only on ADC2, 4
  7,  // 14/A0  AD_B1_02
  8,  // 15/A1  AD_B1_03
  12, // 16/A2  AD_B1_07
  11, // 17/A3  AD_B1_06
  6,  // 18/A4  AD_B1_01
  5,  // 19/A5  AD_B1_00
  15, // 20/A6  AD_B1_10
  0,  // 21/A7  AD_B1_11
  13, // 22/A8  AD_B1_08
  14, // 23/A9  AD_B1_09
  0x40+1,  // 24/A10 AD_B0_12
  0x40+2,  // 25/A11 AD_B0_13
  0x80+3,  // 26/A12 AD_B1_14 - only on ADC2, 3
  0x80+4 // 27/A13 AD_B1_15 - only on ADC2, 4
};


extern int T4AnalogRead(uint8_t pin, uint8_t adc_num){
  if (pin > sizeof(t4_pin_to_channel)) return 0;
  // I believe all the calibration should be done by now
  uint8_t ch = t4_pin_to_channel[pin];
  switch (adc_num) {
    case 1: 
      if (ch & 0x80) return 0;  // does not handle this pin
      break;
    case 2:
      if (ch & 0x40) return 0;  // does not handle this pin
      break;
    default:
      adc_num =  (ch & 0x80)? 2 : 1;
  }
  if(adc_num == 1) {
    ADC1_HC0 = ch & 0x3f;
    while (!(ADC1_HS & ADC_HS_COCO0)) ; // wait
    return ADC1_R0;
  } else {
    ADC2_HC0 = ch & 0x3f;
    while (!(ADC2_HS & ADC_HS_COCO0)) ; // wait
    return ADC2_R0;
  }  
}

bool T4AnalogReadStart(uint8_t pin, uint8_t adc_num)
{
  // warning not checking to see if already busy or ...
  if (pin > sizeof(t4_pin_to_channel)) return false;
  // I believe all the calibration should be done by now
  uint8_t ch = t4_pin_to_channel[pin];
  switch (adc_num) {
    case 1: 
      if (ch & 0x80) return false;  // does not handle this pin
      break;
    case 2:
      if (ch & 0x40) return false;  // does not handle this pin
      break;
    default:
      return false;
  }
  if(adc_num == 1) ADC1_HC0 = ch & 0x3f;
  else ADC2_HC0 = ch & 0x3f;
  return true;
}

int T4AnalogReadComplete(uint8_t adc_num, bool wait_complete) {
  if(adc_num == 1) {
    if (!wait_complete && !(ADC1_HS & ADC_HS_COCO0)) return -1;
    while (!(ADC1_HS & ADC_HS_COCO0)) ; // wait
    return ADC1_R0;
  } else {
    if (!wait_complete && !(ADC2_HS & ADC_HS_COCO0)) return -1;
    while (!(ADC2_HS & ADC_HS_COCO0)) ; // wait
    return ADC2_R0;
  }  
}
